/********************************************************/
/*  Document name ..................... CARDPRINT.C     */
/*  Date created  ..................... 05-AUG-88       */
/*  Time created  ..................... 10:00           */
/*  System   .......................... UTIL            */
/*  Revision .......................... 1.00            */
/*  Language .......................... Microsoft C     */
/*  Operating system .................. MSC/DOS 3.xx    */
/*  Compiler/Assembler revision ....... 4.00            */
/*  Author   .......................... Mat*Rat         */
/*  Function .......................... Selectively     */
/*  print some or all addresses from a cardfile.        */
/********************************************************/

/*\r* Revision History:

DD-MMM-YY By Description
--------- -- ------------------------------------------------
05-AUG-88 MR Begin work on print utility for CARDFILE from PC-MAG
08-AUG-88 MR Add 'C'ontinuous or 'Q'uery mode option

*\r*/

/* ---------- System include files -------------------- */

#include <stdio.h>
#include <string.h>
#include <malloc.h>

/* ---------- Module constants ------------------------ */

#define MAXADDRS 256

#define UNDEFINED 127
#define CONTINUOUS  1
#define QUERY       0

#define SHOW1       1
#define SHOW2       2

/* ---------- Module globals -------------------------- */

typedef struct {

char Address[6][32];

  } ADDR_TYPE;

/* ---------- Module procedure prototpyes ------------- */

void process_file ( FILE *, FILE *, char *,
                    ADDR_TYPE *, int *, char *, int);
void output ( ADDR_TYPE **, FILE *, int );
int locate( char *, ADDR_TYPE *);
void put_bar ( FILE * );


/*\p*********************************************/
/* Procedure: main( argc, argv )                */
/* Accept the filename from the command line,   */
/* if not present, prompt user for it.          */
/* Then process that input file, printing all   */
/* or part of the file as the user specifies.   */
/*                                              */
/* Inputs:                                      */
/* argc - int - Count of command line args      */
/* argv - char ** - Command line arguments      */
/*                                              */
/*\p*********************************************/

main( argc, argv )
int argc;
char *argv[];
{ /* main procedure - CARDPRINT */


ADDR_TYPE *Address_Book;
int Address_Count;
int i, mode;
FILE *fPtr, *ofPtr;
char filename[80], outfilename[80], searchstr[33];

Address_Book = (ADDR_TYPE *)malloc(MAXADDRS * sizeof(ADDR_TYPE) );
if (Address_Book == NULL)
  {
  printf("Not enough RAM to run CARDPRINT.\n");
  return;
  }


printf("\nCardPrint - a print utility for CARDFILE from PC-MAG\n");
printf("By M. Ratcliff, 05-AUG-88, Rev. 1.02\n\n");

if (argc > 1)
  strcpy(filename, argv[1]);
else
  {
  printf("CARDFILE data file to print ? ");
  gets(filename);
  }

fPtr = fopen(filename, "r");

if (fPtr == NULL)
  {
  printf("*FILE %s NOT FOUND*\n", filename);
  return;
  }

if (argc > 2)
  {
  strcpy(searchstr, argv[2]);
  i = strcmp(searchstr, "*"); /* asterisk means dump all */
  if (i == 0)
    searchstr[0] = 0;
  }
else
  {
  printf("Search string (return for ALL addresses) ? ");
  gets(searchstr);
  }

i = UNDEFINED;

if (argc > 3)
  i = (int)*argv[3];

do
  {
  mode = i;
  if ( (mode == 'c') || (mode == 'C') )
    mode = CONTINUOUS;
  else if ( (mode == 'q') || (mode == 'Q') )
    mode = QUERY;
  else
    mode = UNDEFINED;
  if (mode == UNDEFINED)
    {
    printf("[C]ontinuous or [Q]uery mode ? ");
    i = getche();
    printf("\n");
    }
  }
while ( mode == UNDEFINED );

if (argc > 4)
  strcpy(outfilename, argv[4]);
else
  {
  printf("Output file for printing ?\n");
  printf("(Return only for default PRN:)\n? ");
  gets(outfilename);
  if (strlen(outfilename)==0)
    strcpy(outfilename, "prn:");
  }

i = strcmpi(outfilename, "con:");

if (i==0)
  ofPtr = stdout;
else
  {
  i = strcmpi(outfilename, "prn:");
  if (i==0)
    ofPtr = stdprn;
  else
    ofPtr = freopen(outfilename, "w", stdprn);
  }

if (ofPtr == NULL)
  {
  printf("Cannot open %s for output!\n\n", outfilename);
  fclose(fPtr);
  return;
  }
printf("Processing card file...\n");

process_file( fPtr, ofPtr, searchstr, Address_Book, &Address_Count,
              filename, mode );

printf("CardPrint complete.\n");

free(Address_Book);

} /* main end */


/*\p*********************************************/
/* Procedure:  process_file                     */
/* All initialization is complete. Read the     */
/* address file and output according to         */
/* predefined parameters and file pointers.     */
/*                                              */
/* Inputs:                                      */
/* fPtr - FILE * - Input file pointer           */
/* ofPtr - FILE * - Output file pointer         */
/* searchstr - char * - Search template string  */
/* addr - ADDR_TYPE * - Address book buffer     */
/* addr_count - INT - Addresses in address book */
/* addrfil - char * - Address book filename     */
/* mode - INT - Continuous or query output ctl. */
/*                                              */
/*\p*********************************************/

void process_file( fPtr, ofPtr, searchstr,
                   addr, addr_count, addrfil, mode )
FILE *fPtr, *ofPtr;
char *searchstr;
ADDR_TYPE *addr;
int *addr_count;
char *addrfil;
int mode;

{ /* process_file procedure */

int i, err, slen, outputs, linecounts;
int found, page, showit;
char ch;
ADDR_TYPE *faddr[2];
ADDR_TYPE *taddr[2];

page = 1;

slen = strlen( searchstr );

if (slen)
  strlwr(searchstr);

fscanf(fPtr,"%c", &ch);
*addr_count = (unsigned int)(ch) & 0x0ff;

printf("Processing %d addresses.\n", *addr_count);

fprintf(ofPtr, "\r\n      -%3.0d -\t\tAddress book: %s\r\n\r\n",page, addrfil );

i = fread(addr, *addr_count, sizeof(ADDR_TYPE), fPtr);

fclose( fPtr );
if (*addr_count)
  {
  put_bar( ofPtr );
  linecounts = 0;
  outputs = 0;
  faddr[0] = NULL;
  faddr[1] = NULL;
  for (i=0; i<*addr_count; i++)
    {
    if (slen)
      found = locate( searchstr, addr );
    if (mode)
      showit = mode;
    else if ( (!slen) || (found) )
      {
      taddr[0] = addr;
      taddr[1] = NULL;
      output(taddr, stdout, SHOW1);
      printf("Keep this address ? (Y/N)");
      showit = getch();
      if ( (showit == 'y') || (showit == 'Y') )
        {
        printf("Yes\n\n");
        showit = 1;
        }
      else
        {
        printf("No\n\n");
        showit = 0;
        }
      }
    if ( (!slen) || (found) && (showit) )
      faddr[outputs++] = addr;
    addr++;
    if (outputs == 2)
      {
      outputs = 0;
      output(faddr, ofPtr, SHOW2);
      put_bar( ofPtr );
      linecounts++;
      if (linecounts == 7)
        {
        page++;
        fprintf(ofPtr, "%c\r\n", (char)12);
        fprintf(ofPtr, "\r\n      -%3.0d -\t\tAddress book: %s\r\n\r\n",
                page,
                addrfil );
        put_bar( ofPtr );
        linecounts = 0;
        }
      }
    }
  }

if (outputs)
  {
  faddr[1] = NULL;
  output(faddr, ofPtr, SHOW2);
  put_bar( ofPtr );
  }

fprintf(ofPtr, "%c", (char)12);

fclose(ofPtr);


} /* process_file end */


/*\p*********************************************/
/* Procedure:   locate                          */
/* Search for the specified string within the   */
/* address record pointed to.                   */
/*                                              */
/* Inputs:                                      */
/* str - char * - Search string                 */
/* addr - ADDR_TYPE * - Address record to search*/
/*                                              */
/* Output:                                      */
/* INT - 1 if string found, 0 if not            */
/*                                              */
/*\p*********************************************/

int locate( str, addr )
char *str;
ADDR_TYPE *addr;
{ /* locate function */

int i, j;
char works[33];
char ad[sizeof(ADDR_TYPE)+1], *loc;

strncpy(ad, (char *)addr, sizeof(ADDR_TYPE) );
ad[sizeof(ADDR_TYPE)] = 0;
strlwr(ad);

loc = strstr(ad, str);

if (loc == NULL)
  return(0);
else
  return(1);

} /* locate end */


/*\p*********************************************/
/* Procedure:   output                          */
/* Output the record specified to the           */
/* output file pointed to based on display type */
/*                                              */
/* Inputs:                                      */
/* record - ADDR_TYPE ** -The address to output */
/* ofPtr - FILE * - Output file pointer         */
/* qty - int - Output 1 (screen) or 2 (file)    */
/*             address records.                 */
/*                                              */
/*                                              */
/*\p*********************************************/

void output ( record, ofPtr, qty )
ADDR_TYPE *record[];
FILE *ofPtr;
int qty;

{ /* output procedure */

char outstr[80];
int i;

for (i=0; i<6; i++)
  {
  record[0]->Address[i][31] = '\0';
  if (record[1] != NULL)
    record[1]->Address[i][31] = '\0';
  }

for (i=0; i<6; i++)
  {
  fprintf(ofPtr, "      | %s | ", record[0]->Address[i] );
  if (record[1] != NULL)
    fprintf(ofPtr, "%s |\r\n", record[1]->Address[i] );
  else if (qty == SHOW2)
    fprintf(ofPtr, "                                |\r\n");
  else
    fprintf(ofPtr,"\r\n");
  }


} /* output end */

void put_bar( ofPtr )
FILE *ofPtr;
{ /* put_bar */

fprintf(ofPtr, "      +---------------------------------+");
fprintf(ofPtr,  "---------------------------------+\r\n");

}
